@@ -1,6 +1,6 @@
 #include <filezilla.h>
 
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "list.h"
 
 enum listStates
@@ -15,24 +15,17 @@ int CStorjListOpData::Send()
 {
 	switch (opState) {
 	case list_init:
-		if (!subDir_.empty()) {
-			log(logmsg::error, _("Invalid path"));
-			return FZ_REPLY_ERROR;
-		}
-
+		path_ = CServerPath::GetChanged(currentPath_, path_, subDir_);
+		subDir_.clear();
 		if (path_.empty()) {
 			path_ = CServerPath(L"/");
 		}
-
 		currentPath_ = path_;
 
-		if (!currentServer_) {
-			log(logmsg::debug_warning, L"CStorjControlSocket::List called with m_pCurrenServer == 0");
-			return FZ_REPLY_INTERNALERROR;
-		}
+		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), currentPath_.GetPath());
 
 		if (currentPath_.GetType() != ServerType::UNIX) {
-			log(logmsg::debug_warning, L"CStorControlSocket::List called with incompatible server type %d in path", currentPath_.GetType());
+			log(logmsg::debug_warning, L"CStorjListOpData::Send called with incompatible server type %d in path", currentPath_.GetType());
 			return FZ_REPLY_INTERNALERROR;
 		}
 
@@ -77,7 +70,7 @@ int CStorjListOpData::Send()
 		}
 	}
 
-	log(logmsg::debug_warning, L"Unknown opState in CStorjListOpData::ListSend()");
+	log(logmsg::debug_warning, L"Unknown opState in CStorjListOpData::Send()");
 	return FZ_REPLY_INTERNALERROR;
 }
 
