@@ -1,9 +1,11 @@
 #include <filezilla.h>
 
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "list.h"
 #include "rmd.h"
 
+#include <assert.h>
+
 enum mkdStates
 {
 	rmd_init = 0,
