@@ -19,10 +19,14 @@
 // requests have to be answered. Once processed, call
 // CFileZillaEngine::SetAsyncRequestReply to continue the current operation.
 
+#include "commands.h"
 #include "local_path.h"
 #include "server.h"
 
+#include <logging.h>
+
 #include <libfilezilla/time.hpp>
+#include <libfilezilla/tls_info.hpp>
 
 class CFileZillaEngine;
 
@@ -39,26 +43,28 @@ enum NotificationId
 	nId_logmsg,				// notification about new messages for the message log
 	nId_operation,			// operation reply codes
 	nId_connection,			// connection information: connects, disconnects, timeouts etc..
-	nId_transferstatus,		// transfer information: bytes transferes, transfer speed and such
+	nId_transferstatus,		// transfer information: bytes transferred, transfer speed and such
 	nId_listing,			// directory listings
 	nId_asyncrequest,		// asynchronous request
 	nId_active,				// sent if data gets either received or sent
 	nId_data,				// for memory downloads, indicates that new data is available.
 	nId_sftp_encryption,	// information about key exchange, encryption algorithms and so on for SFTP
-	nId_local_dir_created	// local directory has been created
+	nId_local_dir_created,	// local directory has been created
+	nId_serverchange		// With some protocols, actual server identity isn't known until after logon
 };
 
 // Async request IDs
 enum RequestId
 {
-	reqId_fileexists,		// Target file already exists, awaiting further instructions
-	reqId_interactiveLogin, // gives a challenge prompt for a password
-	reqId_hostkey,			// used only by SSH/SFTP to indicate new host key
-	reqId_hostkeyChanged,	// used only by SSH/SFTP to indicate changed host key
-	reqId_certificate,		// sent after a successful TLS handshake to allow certificate
-							// validation
-	reqId_insecure_ftp		// If using opportunistic FTP over TLS, ask user whether he really wants
-							// to use plaintext FTP
+	reqId_fileexists,         // Target file already exists, awaiting further instructions
+	reqId_interactiveLogin,   // gives a challenge prompt for a password
+	reqId_hostkey,            // used only by SSH/SFTP to indicate new host key
+	reqId_hostkeyChanged,     // used only by SSH/SFTP to indicate changed host key
+	reqId_certificate,        // sent after a successful TLS handshake to allow certificate
+	                          // validation.
+	reqId_insecure_connection // If using opportunistic FTP over TLS, or a completely
+	                          // unprotected protocol ask user whether he really wants
+	                          // to use a plaintext connection.
 };
 
 class CNotification
@@ -88,19 +94,21 @@ protected:
 class CLogmsgNotification final : public CNotificationHelper<nId_logmsg>
 {
 public:
-	explicit CLogmsgNotification(MessageType t)
+	explicit CLogmsgNotification(logmsg::type t)
 		: msgType(t)
 	{}
 
 	template<typename String>
-	CLogmsgNotification(MessageType t, String && m)
+	CLogmsgNotification(logmsg::type t, String && m, fz::datetime const& time)
 		: msg(std::forward<String>(m))
+		, time_(time)
 		, msgType(t)
 	{
 	}
 
 	std::wstring msg;
-	MessageType msgType{MessageType::Status}; // Type of message, see logging.h for details
+	fz::datetime time_;
+	logmsg::type msgType{logmsg::status}; // Type of message, see logging.h for details
 };
 
 // If CFileZillaEngine does return with FZ_REPLY_WOULDBLOCK, you will receive
@@ -337,139 +345,17 @@ protected:
 	size_t m_len;
 };
 
-class CCertificate final
-{
-public:
-	class SubjectName final
-	{
-	public:
-		std::wstring name;
-		bool isDns{};
-	};
-
-	CCertificate() = default;
-	CCertificate(CCertificate const& op) = default;
-
-	CCertificate(
-		std::vector<uint8_t> const& rawData,
-		fz::datetime const& activationTime, fz::datetime const& expirationTime,
-		std::wstring const& serial,
-		std::wstring const& pkalgoname, unsigned int bits,
-		std::wstring const& signalgoname,
-		std::wstring const& fingerprint_sha256,
-		std::wstring const& fingerprint_sha1,
-		std::wstring const& issuer,
-		std::wstring const& subject,
-		std::vector<SubjectName> const& altSubjectNames);
-
-	CCertificate(
-		std::vector<uint8_t> && rawdata,
-		fz::datetime const& activationTime, fz::datetime const& expirationTime,
-		std::wstring const& serial,
-		std::wstring const& pkalgoname, unsigned int bits,
-		std::wstring const& signalgoname,
-		std::wstring const& fingerprint_sha256,
-		std::wstring const& fingerprint_sha1,
-		std::wstring const& issuer,
-		std::wstring const& subject,
-		std::vector<SubjectName> && altSubjectNames);
-
-
-	std::vector<uint8_t> GetRawData() const { return raw_cert_; }
-	fz::datetime GetActivationTime() const { return m_activationTime; }
-	fz::datetime GetExpirationTime() const { return m_expirationTime; }
-
-	std::wstring const& GetSerial() const { return m_serial; }
-	std::wstring const& GetPkAlgoName() const { return m_pkalgoname; }
-	unsigned int GetPkAlgoBits() const { return m_pkalgobits; }
-
-	std::wstring const& GetSignatureAlgorithm() const { return m_signalgoname; }
-
-	std::wstring const& GetFingerPrintSHA256() const { return m_fingerprint_sha256; }
-	std::wstring const& GetFingerPrintSHA1() const { return m_fingerprint_sha1; }
-
-	std::wstring const& GetSubject() const { return m_subject; }
-	std::wstring const& GetIssuer() const { return m_issuer; }
-
-	std::vector<SubjectName> const& GetAltSubjectNames() const { return m_altSubjectNames; }
-
-	explicit operator bool() const { return !raw_cert_.empty(); }
-
-private:
-	fz::datetime m_activationTime;
-	fz::datetime m_expirationTime;
-
-	std::vector<uint8_t> raw_cert_;
-
-	std::wstring m_serial;
-	std::wstring m_pkalgoname;
-	unsigned int m_pkalgobits{};
-
-	std::wstring m_signalgoname;
-
-	std::wstring m_fingerprint_sha256;
-	std::wstring m_fingerprint_sha1;
-
-	std::wstring m_issuer;
-	std::wstring m_subject;
-
-	std::vector<SubjectName> m_altSubjectNames;
-};
-
 class CCertificateNotification final : public CAsyncRequestNotification
 {
 public:
-	CCertificateNotification(std::wstring const& host, unsigned int port,
-		std::wstring const& protocol,
-		std::wstring const& keyExchange,
-		std::wstring const& sessionCipher,
-		std::wstring const& sessionMac,
-		int algorithmWarnings,
-		std::vector<CCertificate> && certificates,
-		bool systemTrust,
-		bool hostnameMismatch);
+	CCertificateNotification(fz::tls_session_info && info);
 	virtual RequestId GetRequestID() const { return reqId_certificate; }
 
-	std::wstring const& GetHost() const { return m_host; }
-	unsigned int GetPort() const { return m_port; }
+	fz::tls_session_info info_;
 
-	std::wstring const& GetSessionCipher() const { return m_sessionCipher; }
-	std::wstring const& GetSessionMac() const { return m_sessionMac; }
-
-	bool m_trusted{};
-
-	const std::vector<CCertificate> GetCertificates() const { return m_certificates; }
-
-	std::wstring const& GetProtocol() const { return m_protocol; }
-	std::wstring const& GetKeyExchange() const { return m_keyExchange; }
-
-	enum algorithm_warnings_t
-	{
-		tlsver = 1,
-		cipher = 2,
-		mac = 4,
-		kex = 8
-	};
-
-	int GetAlgorithmWarnings() const { return m_algorithmWarnings; }
-
-	bool SystemTrust() const { return systemTrust_; }
-	bool MismatchedHostname() const { return hostnameMismatch_; }
+	bool trusted_{};
 
 private:
-	std::wstring m_host;
-	unsigned int m_port{};
-
-	std::wstring m_protocol;
-	std::wstring m_keyExchange;
-	std::wstring m_sessionCipher;
-	std::wstring m_sessionMac;
-	int m_algorithmWarnings{};
-
-	std::vector<CCertificate> m_certificates;
-
-	bool systemTrust_{};
-	bool hostnameMismatch_{};
 };
 
 class CSftpEncryptionNotification final : public CNotificationHelper<nId_sftp_encryption>, public CSftpEncryptionDetails
@@ -482,14 +368,26 @@ public:
 	CLocalPath dir;
 };
 
-class CInsecureFTPNotification final : public CAsyncRequestNotification
+class CInsecureConnectionNotification final : public CAsyncRequestNotification
 {
 public:
-	CInsecureFTPNotification(CServer const& server);
-	virtual RequestId GetRequestID() const { return reqId_insecure_ftp; }
+	CInsecureConnectionNotification(CServer const& server);
+	virtual RequestId GetRequestID() const { return reqId_insecure_connection; }
 
 	CServer const server_;
 	bool allow_{};
 };
 
+class ServerChangeNotification final : public CNotificationHelper<nId_serverchange>
+{
+public:
+	ServerChangeNotification() = default;
+
+	explicit ServerChangeNotification(CServer const& server)
+	    : newServer_(server)
+	{}
+
+	CServer newServer_;
+};
+
 #endif
