@@ -28,8 +28,11 @@
 #include "timeformatting.h"
 
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/process.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
+#include <wx/menu.h>
+
 class CLocalListViewDropTarget final : public CFileDropTarget<wxListCtrlEx>
 {
 public:
@@ -415,10 +418,12 @@ regular_dir:
 
 		int num = m_fileData.size();
 		CLocalFileData data;
-		bool wasLink;
+		bool wasLink{};
+		fz::local_filesys::type t{};
 		fz::native_string name;
-		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+		while (local_filesys.get_next_file(name, wasLink, t, &data.size, &data.time, &data.attributes)) {
 			data.name = fz::to_wstring(name);
+			data.dir = t == fz::local_filesys::dir;
 			if (name.empty() || data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
@@ -1908,33 +1913,30 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 			continue;
 		}
 
+
 		wxFileName fn(m_dir.GetPath(), data.name);
 		if (wxLaunchDefaultApplication(fn.GetFullPath(), 0)) {
 			continue;
 		}
-		bool program_exists = false;
-		wxString cmd = GetSystemOpenCommand(fn.GetFullPath(), program_exists);
-		if (cmd.empty()) {
-			auto pos = data.name.find('.');
-			if (pos == std::wstring::npos || (pos == 0 && data.name.find('.', 1) == std::wstring::npos)) {
-				cmd = pEditHandler->GetOpenCommand(fn.GetFullPath(), program_exists);
-			}
+		auto cmd_with_args = GetSystemAssociation(fn.GetFullPath().ToStdWstring());
+		if (cmd_with_args.empty()) {
+			cmd_with_args = pEditHandler->GetAssociation(fn.GetFullPath().ToStdWstring());
 		}
-		if (cmd.empty()) {
+		if (cmd_with_args.empty()) {
 			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 			continue;
 		}
-		if (!program_exists) {
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
+		if (!ProgramExists(cmd_with_args.front())) {
+			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd_with_args.front());
 			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
 			continue;
 		}
 
-		if (wxExecute(cmd)) {
+		if (fz::spawn_detached_process(AssociationToCommand(cmd_with_args, fn.GetFullPath().ToStdWstring()))) {
 			continue;
 		}
-
 		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
+
 	}
 }
 
