@@ -54,7 +54,6 @@ enum interfaceOptions
 	OPTION_SHOW_QUEUE,
 	OPTION_EDIT_DEFAULTEDITOR,
 	OPTION_EDIT_ALWAYSDEFAULT,
-	OPTION_EDIT_INHERITASSOCIATIONS,
 	OPTION_EDIT_CUSTOMASSOCIATIONS,
 	OPTION_COMPARISONMODE,
 	OPTION_COMPARISON_THRESHOLD,
@@ -114,25 +113,28 @@ struct t_OptionsCache
 {
 	bool operator==(std::wstring_view const& v) const { return strValue == v; }
 	bool operator==(int v) const { return numValue == v; }
-	bool operator==(std::unique_ptr<pugi::xml_document> const& v) const { return *xmlValue == *v; }
+	bool operator==(pugi::xml_document const& v) const { return xmlValue == v; }
 	t_OptionsCache& operator=(std::wstring_view const& v);
 	t_OptionsCache& operator=(std::wstring && v);
 	t_OptionsCache& operator=(int v);
-	t_OptionsCache& operator=(std::unique_ptr<pugi::xml_document> const& v);
+	t_OptionsCache& operator=(pugi::xml_document const& v);
+	t_OptionsCache& operator=(pugi::xml_document && v);
 
 	bool from_default;
 	int numValue;
 	std::wstring strValue;
-	std::unique_ptr<pugi::xml_document> xmlValue;
+	pugi::xml_document xmlValue;
 };
 
+std::wstring GetEnv(char const* name);
+
 class CXmlFile;
 class COptions final : public wxEvtHandler, public COptionsBase
 {
 public:
 	virtual int GetOptionVal(unsigned int nID);
 	virtual std::wstring GetOption(unsigned int nID);
-	virtual std::unique_ptr<pugi::xml_document> GetOptionXml(unsigned int nID);
+	virtual pugi::xml_document GetOptionXml(unsigned int nID);
 
 	virtual bool SetOption(unsigned int nID, int value);
 	virtual bool SetOption(unsigned int nID, std::wstring_view const& value);
@@ -158,12 +160,12 @@ protected:
 
 	int Validate(unsigned int nID, int value);
 	std::wstring Validate(unsigned int nID, std::wstring_view const& value);
-	std::unique_ptr<pugi::xml_document> Validate(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value);
+	pugi::xml_document Validate(unsigned int nID, pugi::xml_document const& value);
 
 	template<typename T> void ContinueSetOption(unsigned int nID, T const& value);
-	void SetXmlValue(unsigned int nID, int value);
-	void SetXmlValue(unsigned int nID, std::wstring_view const& value);
-	void SetXmlValue(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value);
+	void SetXmlValue(unsigned int nID, pugi::xml_node settings, int value);
+	void SetXmlValue(unsigned int nID, pugi::xml_node settings, std::wstring_view const& value);
+	void SetXmlValue(unsigned int nID, pugi::xml_node settings, pugi::xml_document const& value);
 
 	pugi::xml_node CreateSettingsXmlElement();
 
@@ -173,6 +175,7 @@ protected:
 	void LoadOptionFromElement(pugi::xml_node option, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault);
 	CLocalPath InitSettingsDir();
 	void SetDefaultValues();
+	void WriteCacheToXml(pugi::xml_node settings);
 
 	bool Cleanup(); // Removes all unknown elements from the XML
 	void Save();
