@@ -6,32 +6,36 @@
 #include "statuslinectrl.h"
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
+#include "file_utils.h"
 #include "ipcmutex.h"
 #include "local_recursive_operation.h"
 #include "state.h"
 #include "asyncrequestqueue.h"
 #include "defaultfileexistsdlg.h"
-#include <wx/dnd.h>
 #include "dndobjects.h"
 #include "loginmanager.h"
 #include "aui_notebook_ex.h"
 #include "queueview_failed.h"
 #include "queueview_successful.h"
 #include "commandqueue.h"
-#include <wx/utils.h>
-#include <wx/progdlg.h>
-#include <wx/sound.h>
 #include "statusbar.h"
 #include "remote_recursive_operation.h"
 #include "auto_ascii_files.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
+
 #if WITH_LIBDBUS
 #include "../dbus/desktop_notification.h"
 #elif defined(__WXGTK__) || defined(__WXMSW__)
 #include <wx/notifmsg.h>
 #endif
 
+#include <wx/dnd.h>
+#include <wx/menu.h>
+#include <wx/progdlg.h>
+#include <wx/sound.h>
+#include <wx/utils.h>
+
 #ifdef __WXMSW__
 #include <powrprof.h>
 #endif
@@ -250,9 +254,9 @@ CQueueView::~CQueueView()
 	m_resize_timer.Stop();
 }
 
-bool CQueueView::QueueFile(const bool queueOnly, const bool download,
+bool CQueueView::QueueFile(bool const queueOnly, bool const download,
 						   std::wstring const& sourceFile, std::wstring const& targetFile,
-						   const CLocalPath& localPath, const CServerPath& remotePath,
+						   CLocalPath const& localPath, CServerPath const& remotePath,
 						   Site const& site, int64_t size, CEditHandler::fileType edit,
 						   QueuePriority priority)
 {
@@ -2941,54 +2945,21 @@ void CQueueView::RenameFileInTransfer(CFileZillaEngine *pEngine, const wxString&
 	RefreshItem(pFile);
 }
 
-std::wstring CQueueView::ReplaceInvalidCharacters(std::wstring const& filename)
+std::wstring CQueueView::ReplaceInvalidCharacters(std::wstring const& filename, bool includeQuotesAndBreaks)
 {
 	if (!COptions::Get()->GetOptionVal(OPTION_INVALID_CHAR_REPLACE_ENABLE)) {
 		return filename;
 	}
 
-	const wxChar replace = COptions::Get()->GetOption(OPTION_INVALID_CHAR_REPLACE)[0];
+	wchar_t const replace = COptions::Get()->GetOption(OPTION_INVALID_CHAR_REPLACE)[0];
 
-	wxString result;
-	{
-		wxStringBuffer start(result, filename.size() + 1);
-		wxChar* buf = start;
-
-		const wxChar* p = filename.c_str();
-		while (*p) {
-			const wxChar c = *p;
-			switch (c)
-			{
-			case '/':
-	#ifdef __WXMSW__
-			case '\\':
-			case ':':
-			case '*':
-			case '?':
-			case '"':
-			case '<':
-			case '>':
-			case '|':
-	#endif
-				if (replace)
-					*buf++ = replace;
-				break;
-			default:
-	#ifdef __WXMSW__
-				if (c < 0x20)
-					*buf++ = replace;
-				else
-	#endif
-				{
-					*buf++ = c;
-				}
-			}
-			p++;
+	std::wstring ret = filename;
+	for (auto & c : ret) {
+		if (IsInvalidChar(c, includeQuotesAndBreaks)) {
+			c = replace;
 		}
-		*buf = 0;
 	}
-
-	return result.ToStdWstring();
+	return ret;
 }
 
 wxFileOffset CQueueView::GetCurrentDownloadSpeed()
