@@ -1,7 +1,7 @@
 #include <filezilla.h>
 #include "customheightlistctrl.h"
 
-IMPLEMENT_DYNAMIC_CLASS(wxCustomHeightListCtrl, wxScrolledWindow)
+#include <algorithm>
 
 BEGIN_EVENT_TABLE(wxCustomHeightListCtrl, wxScrolledWindow)
 EVT_MOUSE_EVENTS(wxCustomHeightListCtrl::OnMouseEvent)
@@ -157,6 +157,11 @@ std::set<size_t> wxCustomHeightListCtrl::GetSelection() const
 	return m_selectedLines;
 }
 
+size_t wxCustomHeightListCtrl::GetRowCount() const
+{
+	return m_rows.size();
+}
+
 void wxCustomHeightListCtrl::SelectLine(size_t line)
 {
 	if (!m_allow_selection) {
@@ -200,6 +205,22 @@ void wxCustomHeightListCtrl::InsertRow(wxSizer* sizer, size_t pos)
 	}
 
 	Refresh();
+
+#ifdef __WXGTK__
+	// Needed for Ubuntu's shitty overlay scrollbar, it changes the layout asynchronously...
+	CallAfter([this](){
+		wxSizeEvent ev;
+		OnSize(ev);
+	});
+#endif
+}
+
+void wxCustomHeightListCtrl::DeleteRow(wxSizer *row)
+{
+	auto it = std::find(m_rows.begin(), m_rows.end(), row);
+	if (it != m_rows.end()) {
+		DeleteRow(it - m_rows.begin());
+	}
 }
 
 void wxCustomHeightListCtrl::DeleteRow(size_t pos)
