@@ -17,6 +17,8 @@ END_EVENT_TABLE()
 std::vector<wxDialogEx*> wxDialogEx::shown_dialogs_;
 
 #ifdef __WXMAC__
+std::vector<void*> wxDialogEx::shown_dialogs_creation_events_;
+
 static int const pasteId = wxNewId();
 static int const selectAllId = wxNewId();
 
@@ -129,6 +131,9 @@ int wxDialogEx::ShowModal()
 #endif
 
 	shown_dialogs_.push_back(this);
+#ifdef __WXMAC__
+	shown_dialogs_creation_events_.push_back(wxGetApp().MacGetCurrentEvent());
+#endif
 
 	if (acceleratorTable_.empty()) {
 		SetAcceleratorTable(wxNullAcceleratorTable);
@@ -139,6 +144,9 @@ int wxDialogEx::ShowModal()
 
 	int ret = wxDialog::ShowModal();
 
+#ifdef __WXMAC__
+	shown_dialogs_creation_events_.pop_back();
+#endif
 	shown_dialogs_.pop_back();
 
 	return ret;
@@ -179,7 +187,8 @@ bool wxDialogEx::CanShowPopupDialog(wxTopLevelWindow * parent)
 #endif
 
 #ifdef __WXMAC__
-	if (wxGetApp().MacGetCurrentEvent()) {
+	void* ev = wxGetApp().MacGetCurrentEvent();
+	if (ev && (shown_dialogs_creation_events_.empty() || ev != shown_dialogs_creation_events_.back())) {
 		// We're inside an event handler for a native mac event, such as a popup menu
 		return false;
 	}
@@ -229,6 +238,7 @@ DialogLayout::DialogLayout(wxTopLevelWindow * parent)
 {
 	gap = dlgUnits(3);
 	border = dlgUnits(3);
+	indent = dlgUnits(10);
 }
 
 int DialogLayout::dlgUnits(int num) const
@@ -327,7 +337,14 @@ std::tuple<wxStaticBox*, wxFlexGridSizer*> DialogLayout::createStatBox(wxSizer*
 	parent->Add(boxSizer, 1, wxGROW);
 
 	auto* flex = createFlex(cols, rows);
-	boxSizer->Add(flex, 1, wxALL|wxGROW, border);
+#ifdef __WXMSW__
+	int style = wxGROW | wxLEFT | wxBOTTOM | wxRIGHT;
+	int const b = dlgUnits(2);
+#else
+	int style = wxGROW | wxALL;
+	int const b = border;
+#endif
+	boxSizer->Add(flex, 1, style, b);
 
 	return std::make_tuple(box, flex);
 }
