@@ -9,6 +9,7 @@ struct DialogLayout final
 public:
 	int gap{};
 	int border{};
+	int indent{};
 
 	int dlgUnits(int num) const;
 
@@ -56,6 +57,10 @@ public:
 
 	DialogLayout const& layout();
 
+	void EndDialog(int rc) {
+		wxDialog::EndDialog(rc);
+	}
+
 protected:
 	virtual void InitDialog();
 
@@ -65,6 +70,8 @@ protected:
 
 #ifdef __WXMAC__
 	virtual bool ProcessEvent(wxEvent& event);
+
+	static std::vector<void*> shown_dialogs_creation_events_;
 #endif
 
 	static std::vector<wxDialogEx*> shown_dialogs_;
