@@ -6,6 +6,8 @@
 #include "queue.h"
 #include "xrc_helper.h"
 
+#include <wx/filedlg.h>
+
 CExportDialog::CExportDialog(wxWindow* parent, CQueueView* pQueueView)
 	: m_parent(parent), m_pQueueView(pQueueView)
 {
@@ -56,7 +58,7 @@ void CExportDialog::Run()
 		return;
 	}
 
-	CXmlFile xml(dlg.GetPath());
+	CXmlFile xml(dlg.GetPath().ToStdWstring());
 
 	auto exportRoot = xml.CreateEmpty();
 
