@@ -8,6 +8,11 @@
 #include "xrc_helper.h"
 
 #include <wx/display.h>
+#include <wx/statbox.h>
+
+#ifndef __WXMSW__
+#include <wx/mimetype.h>
+#endif
 
 BEGIN_EVENT_TABLE(CFileExistsDlg, wxDialogEx)
 EVT_BUTTON(XRCID("wxID_OK"), CFileExistsDlg::OnOK)
@@ -27,7 +32,83 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 
 	SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
 	SetParent(parent);
-	if (!CreateControls()) {
+
+	if (!wxDialogEx::Create(parent, -1, _("Target file already exists"))) {
+		return false;
+	}
+
+	auto& lay = layout();
+	auto* main = lay.createMain(this, 1);
+
+	auto* inner = lay.createFlex(3);
+	main->Add(inner);
+
+	auto* left = lay.createFlex(1);
+	inner->Add(left);
+
+	left->Add(new wxStaticText(this, -1, _("The target file already exists.\nPlease choose an action.")));
+
+	left->AddSpacer(lay.dlgUnits(1));
+
+	left->Add(new wxStaticText(this, -1, _("Source file:")));
+	left->Add(new wxStaticText(this, XRCID("ID_FILE2_NAME"), wxString()));
+	auto* source = lay.createFlex(2);
+	left->Add(source);
+	source->Add(new wxStaticBitmap(this, XRCID("ID_FILE2_ICON"), wxBitmap()), lay.valign);
+	auto* sourceDetails = lay.createFlex(1);
+	source->Add(sourceDetails, lay.valign);
+	sourceDetails->Add(new wxStaticText(this, XRCID("ID_FILE2_SIZE"), wxString()));
+	sourceDetails->Add(new wxStaticText(this, XRCID("ID_FILE2_TIME"), wxString()));
+
+	left->AddSpacer(lay.dlgUnits(1));
+
+	left->Add(new wxStaticText(this, -1, _("Target file:")));
+	left->Add(new wxStaticText(this, XRCID("ID_FILE1_NAME"), wxString()));
+	auto* target = lay.createFlex(2);
+	left->Add(target);
+	target->Add(new wxStaticBitmap(this, XRCID("ID_FILE1_ICON"), wxBitmap()), lay.valign);
+	auto* targetDetails = lay.createFlex(1);
+	target->Add(targetDetails, lay.valign);
+	targetDetails->Add(new wxStaticText(this, XRCID("ID_FILE1_SIZE"), wxString()));
+	targetDetails->Add(new wxStaticText(this, XRCID("ID_FILE1_TIME"), wxString()));
+
+	inner->AddSpacer(lay.indent);
+
+	auto* right = lay.createFlex(1);
+	inner->Add(right);
+
+	auto [box, actions] = lay.createStatBox(right, _("Action:"), 1);
+
+	auto* first = new wxRadioButton(box, XRCID("ID_ACTION1"), _("&Overwrite"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
+	first->SetValue(true);
+	actions->Add(first);
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION2"), _("Overwrite &if source newer")));
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION7"), _("Overwrite if &different size")));
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION6"), _("Overwrite if different si&ze or source newer")));
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION3"), _("&Resume")));
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION4"), _("Re&name")));
+	actions->Add(new wxRadioButton(box, XRCID("ID_ACTION5"), _("&Skip")));
+
+
+	right->Add(new wxCheckBox(this, XRCID("ID_ALWAYS"), _("&Always use this action")));
+	right->Add(new wxCheckBox(this, XRCID("ID_QUEUEONLY"), _("Apply to &current queue only")), 0, wxLEFT, lay.indent);
+	right->Add(new wxCheckBox(this, XRCID("ID_UPDOWNONLY"), wxString()), 0, wxLEFT, lay.indent);
+
+	auto* buttons = lay.createButtonSizer(this, main, true);
+
+	auto ok = new wxButton(this, wxID_OK, _("&OK"));
+	ok->SetDefault();
+	buttons->AddButton(ok);
+
+	auto cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
+	buttons->AddButton(cancel);
+
+	buttons->Realize();
+
+
+
+
+	if (!SetupControls()) {
 		return false;
 	}
 	GetSizer()->Fit(this);
@@ -36,7 +117,7 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	return true;
 }
 
-void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t size, fz::datetime const& time, wxString const& iconFile)
+void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t size, fz::datetime const& time, std::wstring const& iconFile)
 {
 	std::wstring labelName = LabelEscape(GetPathEllipsis(name, FindWindow(left ? XRCID("ID_FILE1_NAME") : XRCID("ID_FILE2_NAME"))));
 
@@ -58,12 +139,8 @@ void CFileExistsDlg::DisplayFile(bool left, std::wstring const& name, int64_t si
 	LoadIcon(left ? XRCID("ID_FILE1_ICON") : XRCID("ID_FILE2_ICON"), iconFile);
 }
 
-bool CFileExistsDlg::CreateControls()
+bool CFileExistsDlg::SetupControls()
 {
-	if (!Load(GetParent(), _T("ID_FILEEXISTSDLG"))) {
-		return false;
-	}
-
 	std::wstring const& localFile = m_pNotification->localFile;
 	std::wstring remoteFile = m_pNotification->remotePath.FormatFilename(m_pNotification->remoteFile);
 
@@ -75,7 +152,7 @@ bool CFileExistsDlg::CreateControls()
 	return true;
 }
 
-void CFileExistsDlg::LoadIcon(int id, const wxString &file)
+void CFileExistsDlg::LoadIcon(int id, std::wstring const& file)
 {
 	wxStaticBitmap *pStatBmp = static_cast<wxStaticBitmap *>(FindWindow(id));
 	if (!pStatBmp) {
@@ -85,12 +162,11 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 	wxSize size = CThemeProvider::GetIconSize(iconSizeNormal);
 	pStatBmp->SetInitialSize(size);
 	pStatBmp->InvalidateBestSize();
-	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
 	memset(&fileinfo, 0, sizeof(fileinfo));
-	if (SHGetFileInfo(file.wc_str(), FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
+	if (SHGetFileInfo(file.c_str(), FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
 		wxBitmap bmp;
 		bmp.Create(size.x, size.y);
 
@@ -116,53 +192,48 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 
 		return;
 	}
+#else
+	std::wstring ext = GetExtension(file);
+	if (!ext.empty() && ext != L".") {
+		std::unique_ptr<wxFileType> pType(wxTheMimeTypesManager->GetFileTypeFromExtension(ext));
+		if (pType) {
+			wxIconLocation loc;
+			if (pType->GetIcon(&loc) && loc.IsOk()) {
+				wxLogNull* tmp = new wxLogNull;
+				wxIcon icon(loc);
+				delete tmp;
+				if (icon.Ok()) {
+					int width = icon.GetWidth();
+					int height = icon.GetHeight();
+					if (width && height) {
+						wxBitmap bmp;
+						bmp.Create(icon.GetWidth(), icon.GetHeight());
 
-#endif //__WXMSW__
+						wxMemoryDC* dc = new wxMemoryDC;
 
-	std::wstring ext = GetExtension(file.ToStdWstring());
-	if (ext.empty()) {
-		return;
-	}
+						wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
+						wxBrush brush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 
-	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-	if (pType) {
-		wxIconLocation loc;
-		if (pType->GetIcon(&loc) && loc.IsOk()) {
-			wxLogNull *tmp = new wxLogNull;
-			wxIcon icon(loc);
-			delete tmp;
-			if (!icon.Ok()) {
-				delete pType;
-				return;
-			}
+						dc->SelectObject(bmp);
 
-			int width = icon.GetWidth();
-			int height = icon.GetHeight();
-			if (width && height) {
-				wxBitmap bmp;
-				bmp.Create(icon.GetWidth(), icon.GetHeight());
+						dc->SetPen(pen);
+						dc->SetBrush(brush);
+						dc->DrawRectangle(0, 0, width, height);
 
-				wxMemoryDC *dc = new wxMemoryDC;
+						dc->DrawIcon(icon, 0, 0);
+						delete dc;
 
-				wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
-				wxBrush brush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
+						pStatBmp->SetBitmap(bmp);
 
-				dc->SelectObject(bmp);
-
-				dc->SetPen(pen);
-				dc->SetBrush(brush);
-				dc->DrawRectangle(0, 0, width, height);
-
-				dc->DrawIcon(icon, 0, 0);
-				delete dc;
-
-				pStatBmp->SetBitmap(bmp);
-
-				return;
+						return;
+					}
+				}
 			}
 		}
-		delete pType;
 	}
+#endif
+
+	pStatBmp->SetBitmap(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, size));
 }
 
 void CFileExistsDlg::OnOK(wxCommandEvent&)
