@@ -9,6 +9,11 @@
 #include <algorithm>
 #include "filelist_statusbar.h"
 #include "themeprovider.h"
+
+#ifndef __WXMSW__
+#include <wx/mimetype.h>
+#endif
+
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
 #include <gtk/gtk.h>
 #endif
@@ -85,8 +90,9 @@ template<class CFileData> WXLRESULT CFileListCtrl<CFileData>::MSWWindowProc(WXUI
 
 template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result)
 {
-	if (!m_pFilelistStatusBar)
+	if (!m_pFilelistStatusBar) {
 		return wxListCtrlEx::MSWOnNotify(idCtrl, lParam, result);
+	}
 
 	*result = 0;
 
@@ -94,31 +100,36 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 	if (pNmhdr->code == LVN_ODSTATECHANGED) {
 		// A range of items got (de)selected
 
-		if (m_insideSetSelection)
+		if (m_insideSetSelection) {
 			return true;
+		}
 
-		if (!m_pFilelistStatusBar)
+		if (!m_pFilelistStatusBar) {
 			return true;
+		}
 
-		if (wxGetKeyState(WXK_CONTROL) && wxGetKeyState(WXK_SHIFT))
-		{
+		if (wxGetKeyState(WXK_CONTROL) && wxGetKeyState(WXK_SHIFT)) {
 			// The behavior of Ctrl+Shift+Click is highly erratic.
 			// Even though it is very slow, we need to manually recount.
 			m_pFilelistStatusBar->UnselectAll();
 			int item = -1;
 			while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
-				if (m_hasParent && !item)
+				if (m_hasParent && !item) {
 					continue;
+				}
 
 				const int index = m_indexMapping[item];
 				const CFileData& data = m_fileData[index];
-				if (data.comparison_flags == fill)
+				if (data.comparison_flags == fill) {
 					continue;
+				}
 
-				if (ItemIsDir(index))
+				if (ItemIsDir(index)) {
 					m_pFilelistStatusBar->SelectDirectory();
-				else
+				}
+				else {
 					m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
+				}
 			}
 		}
 		else {
@@ -126,31 +137,35 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 
 			wxASSERT(pNmOdStateChange->iFrom <= pNmOdStateChange->iTo);
 			for (int i = pNmOdStateChange->iFrom; i <= pNmOdStateChange->iTo; ++i) {
-				if (m_hasParent && !i)
+				if (m_hasParent && !i) {
 					continue;
+				}
 
 				const int index = m_indexMapping[i];
 				const CFileData& data = m_fileData[index];
-				if (data.comparison_flags == fill)
+				if (data.comparison_flags == fill) {
 					continue;
+				}
 
-				if (ItemIsDir(index))
+				if (ItemIsDir(index)) {
 					m_pFilelistStatusBar->SelectDirectory();
-				else
+				}
+				else {
 					m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
+				}
 			}
 		}
 		return true;
 	}
 	else if (pNmhdr->code == LVN_ITEMCHANGED) {
-		if (m_insideSetSelection)
+		if (m_insideSetSelection) {
 			return true;
+		}
 
 		NMLISTVIEW* pNmListView = (NMLISTVIEW*)lParam;
 
 		// Item of -1 means change applied to all items
-		if (pNmListView->iItem == -1 && !(pNmListView->uNewState & LVIS_SELECTED))
-		{
+		if (pNmListView->iItem == -1 && !(pNmListView->uNewState & LVIS_SELECTED)) {
 			m_pFilelistStatusBar->UnselectAll();
 		}
 	}
@@ -174,14 +189,17 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 		}
 
 		if (lvi.mask & LVIF_IMAGE) {
-			if (!lvi.iSubItem)
+			if (!lvi.iSubItem) {
 				lvi.iImage = OnGetItemImage(item);
-			else
+			}
+			else {
 				lvi.iImage = -1;
+			}
 		}
 
-		if (!lvi.iSubItem)
+		if (!lvi.iSubItem) {
 			lvi.state = INDEXTOOVERLAYMASK(GetOverlayIndex(lvi.iItem));
+		}
 
 		return true;
 	}
@@ -221,8 +239,9 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 	GdkEventButton* button_event = (GdkEventButton*)gdk_event;
 
 	// 8 is back, 9 is forward.
-	if (button_event->button != 8 && button_event->button != 9)
+	if (button_event->button != 8 && button_event->button != 9) {
 		return FALSE;
+	}
 
 	proxy->OnNavigationEvent(button_event->button == 9);
 
@@ -231,31 +250,21 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 }
 #endif
 
-template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CQueueView* pQueue, bool border /*=false*/)
-: wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER)),
-	CComparableListing(this)
+template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CQueueView* pQueue, bool border)
+	: wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER))
+	, CComparableListing(this)
+	, m_pQueue(pQueue)
 {
 	CreateSystemImageList(CThemeProvider::GetIconSize(iconSizeSmall).x);
-	m_pQueue = pQueue;
-
-	m_hasParent = true;
-
-	m_comparisonIndex = -1;
 
 #ifndef __WXMSW__
 	m_dropHighlightAttribute.SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNSHADOW));
 #endif
 
-	m_pFilelistStatusBar = 0;
-
-	m_insideSetSelection = false;
 #ifdef __WXMSW__
 	// Enable use of overlay images
 	DWORD mask = ListView_GetCallbackMask((HWND)GetHandle()) | LVIS_OVERLAYMASK;
 	ListView_SetCallbackMask((HWND)GetHandle(), mask);
-#else
-	m_pending_focus_processing = 0;
-	m_focusItem = -1;
 #endif
 
 #if defined(__WXGTK__) && !defined(__WXGTK3__)
@@ -265,8 +274,8 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.get());
 #endif
 
-	m_genericTypes[genericTypes::file] = _("File");
-	m_genericTypes[genericTypes::directory] = _("Directory");
+	m_genericTypes[genericTypes::file] = _("File").ToStdWstring();
+	m_genericTypes[genericTypes::directory] = _("Directory").ToStdWstring();
 
 	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 #ifndef __WXMSW__
@@ -274,10 +283,6 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 #endif
 }
 
-template<class CFileData> CFileListCtrl<CFileData>::~CFileListCtrl()
-{
-}
-
 template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=-1*/, int direction /*=-1*/, bool updateSelections /*=true*/)
 {
 	CancelLabelEdit();
@@ -285,8 +290,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	if (column != -1) {
 		if (column != m_sortColumn) {
 			const int oldVisibleColumn = GetColumnVisibleIndex(m_sortColumn);
-			if (oldVisibleColumn != -1)
+			if (oldVisibleColumn != -1) {
 				SetHeaderSortIconIndex(oldVisibleColumn, -1);
+			}
 		}
 	}
 	else {
@@ -298,8 +304,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		}
 	}
 
-	if (direction == -1)
+	if (direction == -1) {
 		direction = m_sortDirection;
+	}
 
 
 	if (column != m_sortColumn || direction != m_sortDirection) {
@@ -326,7 +333,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 			if (item != -1) {
 				selected = new bool[m_fileData.size()];
 				memset(selected, 0, sizeof(bool) * m_fileData.size());
-			
+
 				do {
 					selected[m_indexMapping[item]] = 1;
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -334,8 +341,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 			}
 		}
 		focused_item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-		if (focused_item != -1)
+		if (focused_item >= 0 && static_cast<size_t>(focused_item) < m_indexMapping.size()) {
 			focused_index = m_indexMapping[focused_item];
+		}
 	}
 
 	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
@@ -347,8 +355,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		m_sortDirection = direction;
 		m_sortColumn = column;
 		std::vector<unsigned int>::iterator start = m_indexMapping.begin();
-		if (m_hasParent)
+		if (m_hasParent) {
 			++start;
+		}
 		std::reverse(start, m_indexMapping.end());
 
 		if (updateSelections) {
@@ -369,11 +378,11 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	}
 
 	std::vector<unsigned int>::iterator start = m_indexMapping.begin();
-	if (m_hasParent)
+	if (m_hasParent) {
 		++start;
-	CSortComparisonObject object = GetSortComparisonObject();
-	std::sort(start, m_indexMapping.end(), object);
-	object.Destroy();
+	}
+	std::unique_ptr<CFileListCtrlSortBase> object = GetSortComparisonObject();
+	std::sort(start, m_indexMapping.end(), SortPredicate(object));
 
 	if (updateSelections) {
 		SortList_UpdateSelections(selected, focused_item, focused_index);
@@ -394,15 +403,16 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 			}
 		}
 	}
-	
+
 	if (selections) {
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
 			const int state = GetItemState(i, wxLIST_STATE_SELECTED);
 			const bool selected = (state & wxLIST_STATE_SELECTED) != 0;
 
 			int item = m_indexMapping[i];
-			if (selections[item] != selected)
+			if (selections[item] != selected) {
 				SetSelection(i, selections[item]);
+			}
 		}
 	}
 }
@@ -419,10 +429,12 @@ template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFile
 		dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
 		break;
 	case 1:
-		if (m_sortDirection)
+		if (m_sortDirection) {
 			dirSortMode = CFileListCtrlSortBase::dirsort_onbottom;
-		else
+		}
+		else {
 			dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
+		}
 		break;
 	case 2:
 		dirSortMode = CFileListCtrlSortBase::dirsort_inline;
@@ -457,8 +469,9 @@ template<class CFileData> CFileListCtrlSortBase::NameSortMode CFileListCtrl<CFil
 template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListEvent &event)
 {
 	int col = m_pVisibleColumnMapping[event.GetColumn()];
-	if (col == -1)
+	if (col == -1) {
 		return;
+	}
 
 	if (IsComparing()) {
 #ifdef __WXMSW__
@@ -469,16 +482,19 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 		dlg.SetTitle(_("Directory comparison"));
 		dlg.AddText(_("Sort order cannot be changed if comparing directories."));
 		dlg.AddText(_("End comparison and change sorting order?"));
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return;
+		}
 		ExitComparisonMode();
 	}
 
 	int dir;
-	if (col == m_sortColumn)
+	if (col == m_sortColumn) {
 		dir = m_sortDirection ? 0 : 1;
-	else
+	}
+	else {
 		dir = m_sortDirection;
+	}
 
 	SortList(col, dir);
 	RefreshListOnly(false);
@@ -491,112 +507,124 @@ wxString GetExt(const wxString& file)
 	wxString ret;
 
 	int pos = file.Find('.', true);
-	if (pos > 0 && (static_cast<size_t>(pos) + 1) < file.size()) // Does neither starts nor end with dot
+	if (pos > 0 && (static_cast<size_t>(pos) + 1) < file.size()) { // Does neither starts nor end with dot
 		ret = file.Mid(pos + 1);
+	}
 
 	return ret;
 }
 }
 #endif
 
-template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString name, bool dir, const wxString& path /*=_T("")*/)
+template<class CFileData> std::wstring CFileListCtrl<CFileData>::GetType(std::wstring name, bool dir, std::wstring const& path)
 {
 #ifdef __WXMSW__
-	wxString ext = GetExt(name);
-	ext.MakeLower();
-	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(ext);
-	if (typeIter != m_fileTypeMap.end())
+	wxString ext;
+	if (dir) {
+		if (!path.empty()) {
+			ext = '/';
+		}
+	}
+	else {
+		ext = GetExt(name);
+		ext.MakeLower();
+	}
+	auto typeIter = m_fileTypeMap.find(ext);
+	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
+	}
 
 	wxString type;
 	int flags = SHGFI_TYPENAME;
-	if (path.empty())
+	if (path.empty()) {
 		flags |= SHGFI_USEFILEATTRIBUTES;
-	else if (path == _T("\\"))
+	}
+	else if (path == _T("\\")) {
 		name += _T("\\");
-	else
+	}
+	else {
 		name = path + name;
+	}
 
 	SHFILEINFO shFinfo;
 	memset(&shFinfo, 0, sizeof(SHFILEINFO));
-	if (SHGetFileInfo(name.wc_str(),
+	if (SHGetFileInfo(name.c_str(),
 		dir ? FILE_ATTRIBUTE_DIRECTORY : FILE_ATTRIBUTE_NORMAL,
 		&shFinfo,
 		sizeof(shFinfo),
 		flags))
 	{
-		if (!*shFinfo.szTypeName)
-		{
-			if (!ext.empty())
-			{
+		if (!*shFinfo.szTypeName) {
+			if (!ext.empty()) {
 				type = ext;
 				type.MakeUpper();
 				type += _T("-");
 				type += _("file");
 			}
-			else
+			else {
 				type = m_genericTypes[genericTypes::file];
+			}
 		}
-		else
-		{
+		else {
 			type = shFinfo.szTypeName;
-			if (!dir && !ext.empty())
-				m_fileTypeMap[ext.MakeLower()] = type;
+			if (!ext.empty()) {
+				m_fileTypeMap[ext.MakeLower()] = type.ToStdWstring();
+			}
 		}
 	}
-	else
-	{
-		if (!ext.empty())
-		{
+	else {
+		if (!ext.empty()) {
 			type = ext;
 			type.MakeUpper();
 			type += _T("-");
 			type += _("file");
 		}
-		else
+		else {
 			type = m_genericTypes[genericTypes::file];
+		}
 	}
-	return type;
+	return type.ToStdWstring();
 #else
 	(void)path;
 
-	if (dir)
+	if (dir) {
 		return m_genericTypes[genericTypes::directory];
+	}
 
-	int pos = name.Find('.', true);
-	if (pos < 1 || !name[pos + 1]) // Starts or ends with dot
+	size_t pos = name.rfind('.');
+	if (pos == std::wstring::npos || pos < 1 || !name[pos + 1]) { // No dot or starts or ends with dot
 		return m_genericTypes[genericTypes::file];
-	wxString ext = name.Mid(pos + 1);
+	}
+	wxString ext = name.substr(pos + 1);
 	wxString lower_ext = ext.Lower();
 
-	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(lower_ext);
-	if (typeIter != m_fileTypeMap.end())
+	auto typeIter = m_fileTypeMap.find(lower_ext);
+	if (typeIter != m_fileTypeMap.end()) {
 		return typeIter->second;
+	}
 
 	wxFileType *pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-	if (!pType)
-	{
+	if (!pType) {
 		wxString desc = ext;
 		desc += _T("-");
 		desc += _("file");
-		m_fileTypeMap[ext] = desc;
-		return desc;
+		m_fileTypeMap[ext] = desc.ToStdWstring();
+		return desc.ToStdWstring();
 	}
 
 	wxString desc;
-	if (pType->GetDescription(&desc) && !desc.empty())
-	{
+	if (pType->GetDescription(&desc) && !desc.empty()) {
 		delete pType;
-		m_fileTypeMap[ext] = desc;
-		return desc;
+		m_fileTypeMap[ext] = desc.ToStdWstring();
+		return desc.ToStdWstring();
 	}
 	delete pType;
 
 	desc = ext;
 	desc += _T("-");
 	desc += _("file");
-	m_fileTypeMap[lower_ext] = desc;
-	return desc;
+	m_fileTypeMap[lower_ext] = desc.ToStdWstring();
+	return desc.ToStdWstring();
 #endif
 }
 
@@ -607,12 +635,14 @@ template<class CFileData> void CFileListCtrl<CFileData>::ScrollTopItem(int item)
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnPostScroll()
 {
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther)
+	if (!pOther) {
 		return;
+	}
 
 	pOther->ScrollTopItem(GetTopItem());
 }
@@ -628,8 +658,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 	m_indexMapping.clear();
 	m_indexMapping.swap(m_originalIndexMapping);
 
-	for (unsigned int i = 0; i < m_fileData.size() - 1; i++)
+	for (unsigned int i = 0; i < m_fileData.size() - 1; ++i) {
 		m_fileData[i].comparison_flags = normal;
+	}
 
 	SetItemCount(m_indexMapping.size());
 
@@ -655,17 +686,20 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 {
 	m_comparisonSelections.clear();
 
-	if (GetItemCount() != (int)m_indexMapping.size())
+	if (GetItemCount() != (int)m_indexMapping.size()) {
 		return;
+	}
 
 	int focus = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 	if (focus != -1) {
 		SetItemState(focus, 0, wxLIST_STATE_FOCUSED);
 		int index = m_indexMapping[focus];
-		if (m_fileData[index].comparison_flags == fill)
+		if (m_fileData[index].comparison_flags == fill) {
 			focus = -1;
-		else
+		}
+		else {
 			focus = index;
+		}
 	}
 	m_comparisonSelections.push_back(focus);
 
@@ -675,11 +709,11 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 #endif
 	{
 		int item = -1;
-		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-		{
+		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 			int index = m_indexMapping[item];
-			if (m_fileData[index].comparison_flags == fill)
+			if (m_fileData[index].comparison_flags == fill) {
 				continue;
+			}
 			m_comparisonSelections.push_back(index);
 		}
 	}
@@ -687,41 +721,40 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 
 template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRestoreSelections()
 {
-	if (m_comparisonSelections.empty())
+	if (m_comparisonSelections.empty()) {
 		return;
+	}
 
 	int focus = m_comparisonSelections.front();
 	m_comparisonSelections.pop_front();
 
 	int item = -1;
-	if (!m_comparisonSelections.empty())
-	{
+	if (!m_comparisonSelections.empty()) {
 		item = m_comparisonSelections.front();
 		m_comparisonSelections.pop_front();
 	}
-	if (focus == -1)
+	if (focus == -1) {
 		focus = item;
+	}
 
-	for (unsigned int i = 0; i < m_indexMapping.size(); i++)
-	{
+	for (unsigned int i = 0; i < m_indexMapping.size(); ++i) {
 		int index = m_indexMapping[i];
-		if (focus == index)
-		{
+		if (focus == index) {
 			SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 			focus = -1;
 		}
 
 		bool isSelected = GetItemState(i, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
 		bool shouldSelected = item == index;
-		if (isSelected != shouldSelected)
+		if (isSelected != shouldSelected) {
 			SetSelection(i, shouldSelected);
+		}
 
-		if (shouldSelected)
-		{
-			if (m_comparisonSelections.empty())
+		if (shouldSelected) {
+			if (m_comparisonSelections.empty()) {
 				item = -1;
-			else
-			{
+			}
+			else {
 				item = m_comparisonSelections.front();
 				m_comparisonSelections.pop_front();
 			}
@@ -743,8 +776,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 	else {
 		m_sortDirection = 0;
 	}
-	if (m_sortDirection < 0 || m_sortDirection > 1)
+	if (m_sortDirection < 0 || m_sortDirection > 1) {
 		m_sortDirection = 0;
+	}
 
 	if (sortInfo.Len() == 3) {
 		m_sortColumn = sortInfo[2] - '0';
@@ -752,8 +786,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 			m_sortColumn = 0;
 		}
 	}
-	else
+	else {
 		m_sortColumn = 0;
+	}
 
 	SetHeaderSortIconIndex(GetColumnVisibleIndex(m_sortColumn), m_sortDirection);
 }
@@ -898,9 +933,9 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 		return;
 	}
 
-	if (old_focus != -1) {
+	if (old_focus >= 0) {
 		bool selected = GetItemState(old_focus, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
-		if (!selected && m_selections[old_focus]) {
+		if (!selected && old_focus < m_selections.size() && m_selections[old_focus]) {
 			// Need to deselect all
 			if (m_pFilelistStatusBar) {
 				m_pFilelistStatusBar->UnselectAll();
@@ -937,6 +972,15 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 
 template <class CFileData> void CFileListCtrl<CFileData>::UpdateSelections(int min, int max)
 {
+	if (min < 0 || min > max) {
+		return;
+	}
+
+	size_t smax = static_cast<size_t>(max);
+	if (m_selections.size() <= smax || m_indexMapping.size() <= smax || m_fileData.size() <= smax) {
+		return;
+	}
+
 	for (int i = min; i <= max; ++i) {
 		bool selected = GetItemState(i, wxLIST_STATE_SELECTED) == wxLIST_STATE_SELECTED;
 		if (selected == m_selections[i]) {
@@ -1023,15 +1067,15 @@ template<class CFileData> void CFileListCtrl<CFileData>::ClearSelection()
 #endif
 	{
 		int item = -1;
-		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-		{
+		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 			SetSelection(item, false);
 		}
 	}
 
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-	if (item != -1)
+	if (item != -1) {
 		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
+	}
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& event)
@@ -1044,14 +1088,19 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 
 	const int code = event.GetKeyCode();
 	const int mods = event.GetModifiers();
-	if (code == 'A' && (mods == wxMOD_CMD || mods == (wxMOD_CONTROL | wxMOD_META))) {
+	if (code == 'A' && (mods & wxMOD_CMD || (mods & (wxMOD_CONTROL | wxMOD_META)) == (wxMOD_CONTROL | wxMOD_META))) {
+		int mask = fill;
+		if (mods & wxMOD_SHIFT) {
+			mask |= normal;
+		}
+
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); ++i) {
 			const CFileData& data = m_fileData[m_indexMapping[i]];
-			if (data.comparison_flags != fill) {
-				SetSelection(i, true);
+			if (data.comparison_flags & mask) {
+				SetSelection(i, false);
 			}
 			else {
-				SetSelection(i, false);
+				SetSelection(i, true);
 			}
 		}
 		if (m_hasParent && GetItemCount()) {
