@@ -1,16 +1,14 @@
-#ifndef __FILEZILLAAPP_H__
-#define __FILEZILLAAPP_H__
-
-#if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
-#include <wx/debugrpt.h>
-#endif
-
-#include <list>
+#ifndef FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
+#define FILEZILLA_INTERFACE_FILEZILLAAPP_HEADER
 
 #include "local_path.h"
 
+#include <vector>
+
 class CWrapEngine;
 class CCommandLine;
+class CThemeProvider;
+
 class CFileZillaApp : public wxApp
 {
 public:
@@ -25,9 +23,12 @@ public:
 	CLocalPath GetDefaultsDir() const { return m_defaultsDir; }
 	CLocalPath GetLocalesDir() const { return m_localesDir; }
 
-	wxString GetSettingsFile(wxString const& name) const;
+	std::wstring GetSettingsFile(std::wstring const& name) const;
 
 	void CheckExistsFzsftp();
+#if ENABLE_STORJ
+	void CheckExistsFzstorj();
+#endif
 
 	void InitLocale();
 	bool SetLocale(int language);
@@ -41,9 +42,11 @@ public:
 	const CCommandLine* GetCommandLine() const { return m_pCommandLine.get(); }
 
 	void ShowStartupProfile();
-	void AddStartupProfileRecord(const wxString& msg);
+	void AddStartupProfileRecord(std::string const& msg);
 
 protected:
+	void CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, char const* env, int setting, std::wstring const& description);
+
 	bool InitDefaultsDir();
 	bool LoadResourceFiles();
 	bool LoadLocales();
@@ -59,22 +62,20 @@ protected:
 	virtual void OnFatalException();
 #endif
 
-	CLocalPath GetDataDir(wxString fileToFind) const;
+	CLocalPath GetDataDir(std::vector<std::wstring> const& fileToFind, std::wstring const& prefixSub, bool searchSelfDir = true) const;
 
-	// FileExists accepts full paths as parameter,
-	// with the addition that path segments may be obmitted
-	// with a wildcard (*). A matching directory will then be searched.
-	// Example: FileExists(_T("/home/*/.filezilla/filezilla.xml"));
-	bool FileExists(const wxString& file) const;
+	bool FileExists(std::wstring const& file) const;
 
 	std::unique_ptr<CWrapEngine> m_pWrapEngine;
 
 	std::unique_ptr<CCommandLine> m_pCommandLine;
 
-	bool m_profilingActive;
-	std::list<std::pair<wxDateTime, wxString> > m_startupProfile;
+	fz::monotonic_clock m_profile_start;
+	std::vector<std::pair<fz::monotonic_clock, std::string>> m_startupProfile;
+
+	std::unique_ptr<CThemeProvider> themeProvider_;
 };
 
 DECLARE_APP(CFileZillaApp)
 
-#endif //__FILEZILLAAPP_H__
+#endif
