@@ -5,6 +5,9 @@
 #include "inputdialog.h"
 
 #include <libfilezilla/process.hpp>
+#include <libfilezilla/translate.hpp>
+
+#include <wx/filedlg.h>
 
 CFZPuttyGenInterface::CFZPuttyGenInterface(wxWindow* parent)
 	: m_parent(parent)
@@ -62,20 +65,20 @@ int CFZPuttyGenInterface::IsKeyFileEncrypted()
 bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::wstring& comment, std::wstring& data)
 {
 	if (!LoadProcess(silent)) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 
 	int needs_conversion = NeedsConversion(keyFile, silent);
 	if (needs_conversion < 0) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 
 	ReplyCode code;
 	if (needs_conversion) {
 		if (silent) {
-			comment = _("Could not load key file");
+			comment = fztranslate("Could not load key file");
 			return false;
 		}
 
@@ -92,12 +95,10 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 
 		msg = wxString::Format(_("Enter the password for the file '%s'.\nThe converted file will be protected with the same password."), keyFile);
 		CInputDialog dlg;
-		if (!dlg.Create(m_parent, _("Password required"), msg)) {
+		if (!dlg.Create(m_parent, _("Password required"), msg, -1, true)) {
 			return false;
 		}
 
-		dlg.SetPasswordMode(true);
-
 		if (dlg.ShowModal() != wxID_OK) {
 			return false;
 		}
@@ -139,14 +140,14 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 		}
 		keyFile = newName;
 	}
-	
+
 	if (!Send(L"fingerprint")) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		return false;
 	}
 	code = GetReply(data);
 	if (code != success) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		data.clear();
 		return false;
 	}
@@ -154,7 +155,7 @@ bool CFZPuttyGenInterface::LoadKeyFile(std::wstring& keyFile, bool silent, std::
 	Send(L"comment");
 	code = GetReply(comment);
 	if (code != success) {
-		comment = _("Could not load key file");
+		comment = fztranslate("Could not load key file");
 		data.clear();
 		comment.clear();
 		return false;
