@@ -1,11 +1,13 @@
-#ifndef __MENU_BAR_H__
-#define __MENU_BAR_H__
+#ifndef FILEZILLA_INTERFACE_MENU_BAR_HEADER
+#define FILEZILLA_INTERFACE_MENU_BAR_HEADER
 
 #include <option_change_event_handler.h>
 #include "state.h"
 
+#include <wx/menu.h>
+
 class CMainFrame;
-class CMenuBar : public wxMenuBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
+class CMenuBar final : public wxMenuBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
 {
 public:
 	CMenuBar();
@@ -18,7 +20,7 @@ public:
 
 	void UpdateBookmarkMenu();
 
-	std::list<int> m_bookmark_menu_ids;
+	std::vector<int> m_bookmark_menu_ids;
 	std::map<int, wxString> m_bookmark_menu_id_map_global;
 	std::map<int, wxString> m_bookmark_menu_id_map_site;
 
@@ -28,7 +30,7 @@ protected:
 
 	void UpdateSpeedLimitMenuItem();
 
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	DECLARE_DYNAMIC_CLASS(CMenuBar)
@@ -39,4 +41,4 @@ protected:
 	std::map<wxMenu*, std::map<int, wxMenuItem*> > m_hidden_items;
 };
 
-#endif //__MENU_BAR_H__
+#endif
