@@ -3,9 +3,11 @@
 
 #include "filezillaapp.h"
 #include "ipcmutex.h"
+#include "Options.h"
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
 
+#include <wx/dirdlg.h>
 #include <wx/osx/core/cfstring.h>
 
 OSXSandboxUserdirs::OSXSandboxUserdirs()
@@ -119,7 +121,7 @@ void OSXSandboxUserdirs::Load()
 		error = _("Access to some local directories could not be restored:") + _T("\n") + error;
 		error += L"\n\n";
 		error += _("Please check the granted permissions and re-add any missing local directories you are working with in the dialog that follows.");
-		wxMessageBox(error, _("Could not restore directory access"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(error, _("Could not restore directory access"), wxICON_EXCLAMATION);
 		OSXSandboxUserdirsDialog dlg;
 		dlg.Run(0);
 	}
@@ -155,7 +157,8 @@ bool OSXSandboxUserdirs::Save()
 
 bool OSXSandboxUserdirs::Add()
 {
-	wxDirDialog dlg(0, (L"Select local data directory"), L"TEST", wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST);
+	std::wstring home = GetEnv("HOME");
+	wxDirDialog dlg(0, (L"Select local data directory"), home, wxDD_DEFAULT_STYLE|wxDD_DIR_MUST_EXIST);
 	if (dlg.ShowModal() != wxID_OK) {
 		return false;
 	}
@@ -164,7 +167,7 @@ bool OSXSandboxUserdirs::Add()
 	wxCFStringRef pathref(path);
 	wxCFRef<CFURLRef> url(CFURLCreateWithFileSystemPath(0, pathref.get(), kCFURLPOSIXPathStyle, true));
 	if (!url) {
-		wxMessageBox(wxString::Format(_("Could not create CFURL from path %s"), path));
+		wxMessageBoxEx(wxString::Format(_("Could not create CFURL from path %s"), path));
 		return false;
 	}
 
@@ -173,13 +176,13 @@ bool OSXSandboxUserdirs::Add()
 	if (!bookmark) {
 		wxString error;
 		append(error, errorRef, L"CFURLCreateBookmarkData");
-		wxMessageBox(_("Could not create security-scoped bookmark from URL:") + error);
+		wxMessageBoxEx(_("Could not create security-scoped bookmark from URL:") + error);
 		return false;
 	}
 
 	std::wstring actualPath = GetPath(url.get());
 	if (actualPath.empty()) {
-		wxMessageBox(_("Could not get path from URL"));
+		wxMessageBoxEx(_("Could not get path from URL"));
 		return false;
 	}
 
@@ -207,7 +210,7 @@ bool OSXSandboxUserdirs::AddFile(std::wstring const& file)
 	wxCFStringRef pathref(file);
 	wxCFRef<CFURLRef> url(CFURLCreateWithFileSystemPath(0, pathref.get(), kCFURLPOSIXPathStyle, true));
 	if (!url) {
-		wxMessageBox(wxString::Format(_("Could not create CFURL from path %s"), file));
+		wxMessageBoxEx(wxString::Format(_("Could not create CFURL from path %s"), file));
 		return false;
 	}
 
@@ -216,13 +219,13 @@ bool OSXSandboxUserdirs::AddFile(std::wstring const& file)
 	if (!bookmark) {
 		wxString error;
 		append(error, errorRef, L"CFURLCreateBookmarkData");
-		wxMessageBox(_("Could not create security-scoped bookmark from URL:") + error);
+		wxMessageBoxEx(_("Could not create security-scoped bookmark from URL:") + error);
 		return false;
 	}
 
 	std::wstring actualPath = GetPath(url.get());
 	if (actualPath.empty()) {
-		wxMessageBox(_("Could not get path from URL"));
+		wxMessageBoxEx(_("Could not get path from URL"));
 		return false;
 	}
 
@@ -275,7 +278,7 @@ void OSXSandboxUserdirsDialog::Run(wxWindow* parent, bool initial)
 
 	XRCCTRL(*this, "wxID_OK", wxButton)->Bind(wxEVT_BUTTON, [initial](wxCommandEvent& evt) {
 		if (initial && OSXSandboxUserdirs::Get().GetDirs().empty()) {
-			wxMessageBox(_("Please add at least one directory you want to download files into or to upload files from."), _("No directory added"));
+			wxMessageBoxEx(_("Please add at least one directory you want to download files into or to upload files from."), _("No directory added"));
 		}
 		else {
 			evt.Skip();
