@@ -3,6 +3,8 @@
 #include "queueview_failed.h"
 #include "edithandler.h"
 
+#include <wx/menu.h>
+
 BEGIN_EVENT_TABLE(CQueueViewFailed, CQueueViewBase)
 EVT_CONTEXT_MENU(CQueueViewFailed::OnContextMenu)
 EVT_MENU(XRCID("ID_REMOVEALL"), CQueueViewFailed::OnRemoveAll)
@@ -15,9 +17,7 @@ END_EVENT_TABLE()
 CQueueViewFailed::CQueueViewFailed(CQueue* parent, int index)
 	: CQueueViewBase(parent, index, _("Failed transfers"))
 {
-	std::list<ColumnId> extraCols;
-	extraCols.push_back(colTime);
-	extraCols.push_back(colErrorReason);
+	std::vector<ColumnId> extraCols({colTime, colErrorReason});
 	CreateColumns(extraCols);
 }
 
@@ -28,9 +28,14 @@ CQueueViewFailed::CQueueViewFailed(CQueue* parent, int index, const wxString& ti
 
 void CQueueViewFailed::OnContextMenu(wxContextMenuEvent&)
 {
-	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_QUEUE_FAILED"));
-	if (!pMenu)
-		return;
+	wxMenu menu;
+	menu.Append(XRCID("ID_REMOVEALL"), _("Remove &all"));
+	menu.Append(XRCID("ID_REQUEUEALL"), _("&Reset and requeue all"));
+
+	menu.AppendSeparator();
+	menu.Append(XRCID("ID_REMOVE"), _("Remove &selected"));
+	menu.Append(XRCID("ID_REQUEUE"), _("R&eset and requeue selected files"));
+	menu.Append(XRCID("ID_EXPORT"), _("E&xport..."));
 
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
@@ -39,12 +44,12 @@ void CQueueViewFailed::OnContextMenu(wxContextMenuEvent&)
 	const bool has_selection = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1;
 #endif
 
-	pMenu->Enable(XRCID("ID_REMOVE"), has_selection);
-	pMenu->Enable(XRCID("ID_REQUEUE"), has_selection);
-	pMenu->Enable(XRCID("ID_REQUEUEALL"), !m_serverList.empty());
+	menu.Enable(XRCID("ID_REMOVE"), has_selection);
+	menu.Enable(XRCID("ID_REQUEUE"), has_selection);
+	menu.Enable(XRCID("ID_REQUEUEALL"), !m_serverList.empty());
+	menu.Enable(XRCID("ID_EXPORT"), GetItemCount() != 0);
 
-	PopupMenu(pMenu);
-	delete pMenu;
+	PopupMenu(&menu);
 }
 
 void CQueueViewFailed::OnRemoveAll(wxCommandEvent&)
@@ -56,16 +61,19 @@ void CQueueViewFailed::OnRemoveAll(wxCommandEvent&)
 	{
 		// First, clear all selections
 		int item;
-		while ((item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
+		while ((item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 			SetItemState(item, 0, wxLIST_STATE_SELECTED);
+		}
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
-	if (pEditHandler)
+	if (pEditHandler) {
 		pEditHandler->RemoveAll(CEditHandler::upload_and_remove_failed);
+	}
 
-	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
 		delete *iter;
+	}
 	m_serverList.clear();
 
 	m_itemCount = 0;
@@ -76,25 +84,27 @@ void CQueueViewFailed::OnRemoveAll(wxCommandEvent&)
 
 	RefreshListOnly();
 
-	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
+	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount()) {
 		m_pQueue->SetSelection(0);
+	}
 }
 
 void CQueueViewFailed::OnRemoveSelected(wxCommandEvent&)
 {
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	if (!GetSelectedItemCount())
+	if (!GetSelectedItemCount()) {
 		return;
+	}
 #endif
 
 	std::list<CQueueItem*> selectedItems;
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		selectedItems.push_front(GetQueueItem(item));
 		SetItemState(item, 0, wxLIST_STATE_SELECTED);
@@ -102,47 +112,44 @@ void CQueueViewFailed::OnRemoveSelected(wxCommandEvent&)
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
 
-	while (!selectedItems.empty())
-	{
+	while (!selectedItems.empty()) {
 		CQueueItem* pItem = selectedItems.front();
 		selectedItems.pop_front();
 
 		CQueueItem* pTopLevelItem = pItem->GetTopLevelItem();
 
-		if (pItem->GetType() == QueueItemType::Server)
-		{
+		if (pItem->GetType() == QueueItemType::Server) {
 			CServerItem* pServerItem = (CServerItem*)pItem;
-			if (pEditHandler && pEditHandler->GetFileCount(CEditHandler::remote, CEditHandler::upload_and_remove_failed, &pServerItem->GetServer()))
-				pEditHandler->RemoveAll(CEditHandler::upload_and_remove_failed, &pServerItem->GetServer());
+			if (pEditHandler && pEditHandler->GetFileCount(CEditHandler::remote, CEditHandler::upload_and_remove_failed, pServerItem->GetSite())) {
+				pEditHandler->RemoveAll(CEditHandler::upload_and_remove_failed, pServerItem->GetSite());
+			}
 		}
-		else if (pItem->GetType() == QueueItemType::File)
-		{
+		else if (pItem->GetType() == QueueItemType::File) {
 			CFileItem* pFileItem = (CFileItem*)pItem;
-			if (pFileItem->m_edit == CEditHandler::remote && pEditHandler)
-			{
-				if (pFileItem->m_edit == CEditHandler::local)
-				{
-					wxString fullPath(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile());
+			if (pFileItem->m_edit == CEditHandler::remote && pEditHandler) {
+				if (pFileItem->m_edit == CEditHandler::local) {
+					std::wstring fullPath(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile());
 					CEditHandler::fileState state = pEditHandler->GetFileState(fullPath);
-					if (state == CEditHandler::upload_and_remove_failed)
+					if (state == CEditHandler::upload_and_remove_failed) {
 						pEditHandler->Remove(fullPath);
+					}
 				}
-				else
-				{
+				else {
 					CServerItem* pServerItem = (CServerItem*)pFileItem->GetTopLevelItem();
-					CEditHandler::fileState state = pEditHandler->GetFileState(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetServer());
-					if (state == CEditHandler::upload_and_remove_failed)
-						pEditHandler->Remove(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetServer());
+					CEditHandler::fileState state = pEditHandler->GetFileState(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetSite());
+					if (state == CEditHandler::upload_and_remove_failed) {
+						pEditHandler->Remove(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetSite());
+					}
 				}
 			}
 		}
 
-		if (!pTopLevelItem->GetChild(1))
-		{
+		if (!pTopLevelItem->GetChild(1)) {
 			// Parent will get deleted
 			// If next selected item is parent, remove it from list
-			if (!selectedItems.empty() && selectedItems.front() == pTopLevelItem)
+			if (!selectedItems.empty() && selectedItems.front() == pTopLevelItem) {
 				selectedItems.pop_front();
+			}
 		}
 		RemoveItem(pItem, true, false, false);
 	}
@@ -150,8 +157,9 @@ void CQueueViewFailed::OnRemoveSelected(wxCommandEvent&)
 	SaveSetItemCount(m_itemCount);
 	RefreshListOnly();
 
-	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
+	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount()) {
 		m_pQueue->SetSelection(0);
+	}
 }
 
 bool CQueueViewFailed::RequeueFileItem(CFileItem* pFileItem, CServerItem* pServerItem)
@@ -159,7 +167,7 @@ bool CQueueViewFailed::RequeueFileItem(CFileItem* pFileItem, CServerItem* pServe
 	CQueueView* pQueueView = m_pQueue->GetQueueView();
 
 	pFileItem->m_errorCount = 0;
-	pFileItem->SetStatusMessage(CFileItem::none);
+	pFileItem->SetStatusMessage(CFileItem::Status::none);
 
 	if (!pFileItem->Download() && pFileItem->GetType() != QueueItemType::Folder && !wxFileName::FileExists(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile())) {
 		delete pFileItem;
@@ -172,21 +180,19 @@ bool CQueueViewFailed::RequeueFileItem(CFileItem* pFileItem, CServerItem* pServe
 			delete pFileItem;
 			return false;
 		}
-		CEditHandler::fileState state = pEditHandler->GetFileState(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetServer());
+		CEditHandler::fileState state = pEditHandler->GetFileState(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetSite());
 		if (state == CEditHandler::unknown) {
 			wxASSERT(pFileItem->Download());
-			std::wstring file = pFileItem->GetRemoteFile();
-			if (!pEditHandler->AddFile(CEditHandler::remote, file, pFileItem->GetRemotePath(), pServerItem->GetServer())) {
-				delete pFileItem;
-				return false;
-			}
-			pFileItem->SetTargetFile(file);
+			pEditHandler->AddFile(CEditHandler::remote, pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile(), pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetSite(), pFileItem->GetSize());
+			delete pFileItem;
+			return true;
 		}
 		else if (state == CEditHandler::upload_and_remove_failed) {
 			wxASSERT(!pFileItem->Download());
 			bool ret = true;
-			if (!pEditHandler->UploadFile(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetServer(), true))
+			if (!pEditHandler->UploadFile(pFileItem->GetRemoteFile(), pFileItem->GetRemotePath(), pServerItem->GetSite(), true)) {
 				ret = false;
+			}
 			delete pFileItem;
 			return ret;
 		}
@@ -208,7 +214,7 @@ bool CQueueViewFailed::RequeueServerItem(CServerItem* pServerItem)
 
 	CQueueView* pQueueView = m_pQueue->GetQueueView();
 
-	CServerItem* pTargetServerItem = pQueueView->CreateServerItem(pServerItem->GetServer());
+	CServerItem* pTargetServerItem = pQueueView->CreateServerItem(pServerItem->GetSite());
 
 	unsigned int childrenCount = pServerItem->GetChildrenCount(false);
 	for (unsigned int i = 0; i < childrenCount; ++i) {
@@ -223,11 +229,13 @@ bool CQueueViewFailed::RequeueServerItem(CServerItem* pServerItem)
 
 	std::vector<CServerItem*>::iterator iter;
 	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter) {
-		if (*iter == pServerItem)
+		if (*iter == pServerItem) {
 			break;
+		}
 	}
-	if (iter != m_serverList.end())
+	if (iter != m_serverList.end()) {
 		m_serverList.erase(iter);
+	}
 
 	delete pServerItem;
 
@@ -243,8 +251,9 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent&)
 {
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	if (!GetSelectedItemCount())
+	if (!GetSelectedItemCount()) {
 		return;
+	}
 #endif
 
 	bool failedToRequeueAll = false;
@@ -253,20 +262,24 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent&)
 	long skipTo = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 		SetItemState(item, 0, wxLIST_STATE_SELECTED);
-		if (item < skipTo)
+		if (item < skipTo) {
 			continue;
+		}
 
 		CQueueItem* pItem = GetQueueItem(item);
-		if (pItem->GetType() == QueueItemType::Server)
+		if (pItem->GetType() == QueueItemType::Server) {
 			skipTo = item + pItem->GetChildrenCount(true) + 1;
+		}
 		selectedItems.push_back(GetQueueItem(item));
 	}
 
-	if (selectedItems.empty())
+	if (selectedItems.empty()) {
 		return;
+	}
 
 	CQueueView* pQueueView = m_pQueue->GetQueueView();
 
@@ -282,7 +295,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent&)
 			CFileItem* pFileItem = (CFileItem*)pItem;
 
 			CServerItem* pOldServerItem = (CServerItem*)pItem->GetTopLevelItem();
-			CServerItem* pServerItem = pQueueView->CreateServerItem(pOldServerItem->GetServer());
+			CServerItem* pServerItem = pQueueView->CreateServerItem(pOldServerItem->GetSite());
 			RemoveItem(pItem, false, false, false);
 
 			failedToRequeueAll |= !RequeueFileItem(pFileItem, pServerItem);
@@ -297,52 +310,59 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent&)
 
 	pQueueView->CommitChanges();
 
-	if (pQueueView->IsActive())
+	if (pQueueView->IsActive()) {
 		pQueueView->AdvanceQueue(false);
+	}
 
 	DisplayNumberQueuedFiles();
 	SaveSetItemCount(m_itemCount);
 	RefreshListOnly();
 
-	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
+	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount()) {
 		m_pQueue->SetSelection(0);
+	}
 
-	if (failedToRequeueAll)
+	if (failedToRequeueAll) {
 		wxMessageBoxEx(_("Not all items could be requeued for transfer."));
+	}
 }
 
 void CQueueViewFailed::OnRequeueAll(wxCommandEvent&)
 {
 	bool ret = true;
-	while (!m_serverList.empty())
+	while (!m_serverList.empty()) {
 		ret &= RequeueServerItem(m_serverList.front());
+	}
 
 	m_fileCountChanged = true;
 
 	CQueueView* pQueueView = m_pQueue->GetQueueView();
 	pQueueView->CommitChanges();
 
-	if (pQueueView->IsActive())
+	if (pQueueView->IsActive()) {
 		pQueueView->AdvanceQueue(false);
+	}
 
 	DisplayNumberQueuedFiles();
 	SaveSetItemCount(m_itemCount);
 	RefreshListOnly();
 
-	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount())
+	if (!m_itemCount && m_pQueue->GetQueueView()->GetItemCount()) {
 		m_pQueue->SetSelection(0);
+	}
 
-	if (!ret)
+	if (!ret) {
 		wxMessageBoxEx(_("Not all items could be requeued for transfer."));
+	}
 }
 
 void CQueueViewFailed::OnChar(wxKeyEvent& event)
 {
-	if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE)
-	{
+	if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE) {
 		wxCommandEvent cmdEvt;
 		OnRemoveSelected(cmdEvt);
 	}
-	else
+	else {
 		event.Skip();
+	}
 }
