@@ -52,6 +52,11 @@ CState* CContextManager::CreateState(CMainFrame &mainFrame)
 
 	NotifyHandlers(pState, STATECHANGE_NEWCONTEXT, _T(""), 0);
 
+	if (!pState->CreateEngine()) {
+		DestroyState(pState);
+		return nullptr;
+	}
+
 	return pState;
 }
 
@@ -500,7 +505,8 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 {
 	if (m_site) {
 		if (site == m_site) {
-			// Nothing changes
+			// Nothing changes, other than possibly credentials
+			m_site = site;
 			return;
 		}
 
@@ -957,15 +963,16 @@ bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 			continue;
 		}
 
-		bool is_dir, is_link;
+		bool is_link{};
+		fz::local_filesys::type t{};
 		fz::native_string file;
-		while (fs.get_next_file(file, is_link, is_dir, 0, 0, 0)) {
+		while (fs.get_next_file(file, is_link, t, 0, 0, 0)) {
 			if (file.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (is_dir) {
+			if (t == fz::local_filesys::dir) {
 				if (is_link) {
 					continue;
 				}
@@ -1331,14 +1338,14 @@ void CState::UpdateSite(std::wstring const& oldPath, Site const& newSite)
 
 	bool changed = false;
 	if (m_site && m_site != newSite) {
-		if (m_site.SitePath() == oldPath && m_site.server == newSite.server) {
+		if (m_site.SitePath() == oldPath && m_site.GetOriginalServer().SameResource(newSite.GetOriginalServer())) {
 			// Update handles
 			m_site.Update(newSite);
 			changed = true;
 		}
 	}
 	if (m_last_site && m_last_site != newSite) {
-		if (m_last_site.SitePath() == oldPath && m_last_site.server == newSite.server) {
+		if (m_last_site.SitePath() == oldPath && m_last_site.GetOriginalServer().SameResource(newSite.GetOriginalServer())) {
 			m_last_site.Update(newSite);
 			if (!m_site) {
 				// Active site has precedence over historic data
@@ -1358,12 +1365,10 @@ void CState::UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> c
 	if (m_site) {
 		for (auto const& active_site : active_sites) {
 			if (active_site.old_path == m_site.SitePath()) {
-				std::unique_ptr<Site> newSite = CSiteManager::GetSiteByPath(active_site.new_path, false).first;
-
-				if (active_site.old_path == m_site.SitePath() && newSite && m_site.SameResource(*newSite)) {
-					if (m_site != *newSite) {
+				if (active_site.old_path == m_site.SitePath() && active_site.site && m_site.GetOriginalServer().SameResource(active_site.site.GetOriginalServer())) {
+					if (m_site != active_site.site) {
 						changed = true;
-						m_site.Update(*newSite);
+						m_site.Update(active_site.site);
 					}
 				}
 				else {
@@ -1378,11 +1383,10 @@ void CState::UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> c
 	else if (m_last_site) {
 		for (auto const& active_site : active_sites) {
 			if (active_site.old_path == m_last_site.SitePath()) {
-				std::unique_ptr<Site> newSite = CSiteManager::GetSiteByPath(active_site.new_path, false).first;
-				if (active_site.old_path == m_last_site.SitePath() && newSite && m_last_site.SameResource(*newSite)) {
-					if (m_last_site != *newSite) {
+				if (active_site.old_path == m_last_site.SitePath() && active_site.site && m_last_site.GetOriginalServer().SameResource(active_site.site.GetOriginalServer())) {
+					if (m_last_site != active_site.site) {
 						changed = true;
-						m_last_site.Update(*newSite);
+						m_last_site.Update(active_site.site);
 					}
 				}
 				else {
@@ -1403,7 +1407,7 @@ void CState::UpdateKnownSites(std::vector<CSiteManagerDialog::_connected_site> c
 void CState::UpdateTitle()
 {
 	if (m_site) {
-		std::wstring const& name = m_site.server.GetName();
+		std::wstring const& name = m_site.GetName();
 		m_title.clear();
 		if (!name.empty()) {
 			m_title = name + _T(" - ");
@@ -1414,3 +1418,13 @@ void CState::UpdateTitle()
 		m_title = _("Not connected");
 	}
 }
+
+void CState::ChangeServer(CServer const& newServer)
+{
+	if (m_site) {
+		if (!m_site.originalServer) {
+			m_site.originalServer = m_site.server;
+		}
+		m_site.server = newServer;
+	}
+}
