@@ -11,6 +11,8 @@
 #include "xrc_helper.h"
 #include "Options.h"
 
+#include <libfilezilla/process.hpp>
+
 #include <wx/animate.h>
 #include <wx/hyperlink.h>
 #include <wx/statline.h>
@@ -84,11 +86,7 @@ int CUpdateDialog::ShowModal()
 	auto buttons = new wxBoxSizer(wxHORIZONTAL);
 	main->Add(buttons, 0, wxALIGN_RIGHT);
 
-	bool debug = false;
-	wxString v;
-	if (wxGetEnv(_T("FZDEBUG"), &v) && v == _T("1")) {
-		debug = true;
-	}
+	bool const debug = GetEnv("FZDEBUG") == L"1";
 	buttons->Add(new wxButton(this, XRCID("ID_DEBUGLOG"), L"show log"))->Show(debug);
 	buttons->Add(new wxButton(this, wxID_CANCEL, _("&Close")));
 
@@ -398,24 +396,29 @@ void CUpdateDialog::UpdaterStateChanged(UpdaterState s, build const& v)
 
 void CUpdateDialog::OnInstall(wxCommandEvent&)
 {
-	wxString f = updater_.DownloadedFile();
-	if( f.empty() ) {
+	std::wstring f = updater_.DownloadedFile();
+	if (f.empty()) {
 		return;
 	}
 	COptions::Get()->SetOption(OPTION_GREETINGRESOURCES, updater_.GetResources());
 #ifdef __WXMSW__
-	wxExecute(_T("\"") + f +  _T("\" /update /NCRC"));
+	std::vector<std::wstring> cmd_with_args;
+	cmd_with_args.push_back(f);
+	cmd_with_args.push_back(L"/update");
+	cmd_with_args.push_back(L"/NCRC");
+	fz::spawn_detached_process(cmd_with_args);
+
 	wxWindow* p = parent_;
-	while( p->GetParent() ) {
+	while (p->GetParent()) {
 		p = p->GetParent();
 	}
 	p->Close();
 #else
-	bool program_exists = false;
-	wxString cmd = GetSystemOpenCommand(f, program_exists);
-	if( program_exists && !cmd.empty() ) {
-		if (wxExecute(cmd))
+	auto cmd_with_args = GetSystemAssociation(f);
+	if (!cmd_with_args.empty()) {
+		if (fz::spawn_detached_process(AssociationToCommand(cmd_with_args, f))) {
 			return;
+		}
 	}
 
 	wxFileName fn(f);
