@@ -1,13 +1,13 @@
-#ifndef __WINDOW_STATE_MANAGER_H__
-#define __WINDOW_STATE_MANAGER_H__
+#ifndef FILEZILLA_INTERFACE_WINDOW_STATE_MANAGER_HEADER
+#define FILEZILLA_INTERFACE_WINDOW_STATE_MANAGER_HEADER
 
 // This class get used to remember toplevel window size and position across
 // sessions.
 
-class CWindowStateManager : public wxEvtHandler
+class CWindowStateManager final : public wxEvtHandler
 {
 public:
-	CWindowStateManager(wxTopLevelWindow* pWindow);
+	explicit CWindowStateManager(wxTopLevelWindow* pWindow);
 	virtual ~CWindowStateManager();
 
 	bool Restore(const unsigned int optionId, const wxSize& default_size = wxSize(-1, -1));
@@ -35,4 +35,4 @@ protected:
 	void OnMove(wxMoveEvent& event);
 };
 
-#endif //__WINDOW_STATE_MANAGER_H__
+#endif
