@@ -44,11 +44,9 @@ void InitHandlers(wxXmlResource& res)
 	res.AddHandler(new wxStaticLineXmlHandler);
 	res.AddHandler(new wxBitmapButtonXmlHandler);
 	res.AddHandler(new wxStaticBitmapXmlHandler);
-	res.AddHandler(new wxListCtrlXmlHandler);
-	res.AddHandler(new wxCheckListBoxXmlHandler);
+	res.AddHandler(new wxHyperlinkCtrlXmlHandler);
 	res.AddHandler(new wxListBoxXmlHandler);
 	res.AddHandler(new wxScrolledWindowXmlHandler);
-	res.AddHandler(new wxHyperlinkCtrlXmlHandler);
 }
 
 namespace {
